""" Views available for the user
"""
from django.contrib import messages
from django.contrib.auth.hashers import make_password
from django.contrib.auth.models import User
from django.core.exceptions import ValidationError
from django.http import HttpResponseRedirect
from django.shortcuts import redirect
from django.template.loader import get_template
from django.urls import reverse

import core_website_app.components.account_request.api as account_request_api
import core_website_app.components.contact_message.api as contact_message_api
import core_website_app.components.help.api as help_api
import core_website_app.components.privacy_policy.api as privacy_policy_api
import core_website_app.components.rules_of_behavior.api as rules_of_behavior_api
import core_website_app.components.terms_of_use.api as terms_of_use_api
from core_main_app.commons.exceptions import ApiError
from core_main_app.commons.validators import validate_password
from core_main_app.utils.markdown_parser import parse
from core_main_app.utils.rendering import render
from core_website_app.components.contact_message.models import ContactMessage
from core_website_app.settings import DISPLAY_NIST_HEADERS
from .forms import RequestAccountForm, ContactForm


def request_new_account(request):
    """Page that allows to request a user account

    Parameters:
        request:

    Returns: Http response
    """
    assets = {
        "js": [
            {"path": "core_website_app/user/js/user_account_req.js", "is_raw": False}
        ],
        "css": ["core_website_app/user/css/list.css"],
    }

    if request.method == "POST":
        request_form = RequestAccountForm(request.POST)
        if request_form.is_valid():
            # Call the API
            try:
                request_form_data = request_form.cleaned_data
                validate_password(request_form_data.get("password1"))

                user = User(
                    username=request_form_data.get("username"),
                    first_name=request_form_data.get("firstname"),
                    last_name=request_form_data.get("lastname"),
                    password=make_password(request_form_data.get("password1")),
                    email=request_form_data.get("email"),
                    is_active=False,
                )

                account_request_api.insert(user)

                messages.add_message(
                    request,
                    messages.INFO,
                    "User Account Request sent to the administrator.",
                )
                return redirect(reverse("core_main_app_homepage"))
            except ApiError as e:
                error_message = str(e)

                error_template = get_template(
                    "core_website_app/user/request_error.html"
                )
                error_box = error_template.render({"error_message": error_message})

                return render(
                    request,
                    "core_website_app/user/request_new_account.html",
                    assets=assets,
                    context={"request_form": request_form, "action_result": error_box},
                )
            except ValidationError as e:
                error_message = "The following error(s) occurred during " "validation:"
                error_items = [str(error) for error in e.messages]

                error_template = get_template(
                    "core_website_app/user/request_error.html"
                )
                error_box = error_template.render(
                    {"error_message": error_message, "error_items": error_items}
                )

                return render(
                    request,
                    "core_website_app/user/request_new_account.html",
                    assets=assets,
                    context={"request_form": request_form, "action_result": error_box},
                )
    else:
        request_form = RequestAccountForm()

    return render(
        request,
        "core_website_app/user/request_new_account.html",
        assets=assets,
        context={"request_form": request_form},
    )


def contact(request):
    """Contact form

    Parameters:
        request:

    Returns: Http response
    """

    if request.method == "POST":
        contact_form = ContactForm(request.POST)
        if contact_form.is_valid():
            # Call the API
            contact_message = ContactMessage(
                name=request.POST["name"],
                email=request.POST["email"],
                content=request.POST["message"],
            )

            contact_message_api.upsert(contact_message)
            messages.add_message(
                request,
                messages.INFO,
                "Your message has been sent to the administrator.",
            )
            return redirect(reverse("core_main_app_homepage"))
    else:
        contact_form = ContactForm()

    return render(
        request,
        "core_website_app/user/contact.html",
        context={"contact_form": contact_form},
    )


def help_page(request):
    """Page that provides FAQ

    Parameters:
        request: Http response

    Returns:
    """
    # Call the API
    help_page_object = help_api.get()
    if help_page_object is not None:
        help_page_object.content = parse(help_page_object.content)

    return render(
        request, "core_website_app/user/help.html", context={"help": help_page_object}
    )


def privacy_policy(request):
    """Page that provides privacy policy

    Parameters:
        request:

    Returns: Http response
    """
    if DISPLAY_NIST_HEADERS:
        return HttpResponseRedirect("https://www.nist.gov/privacy-policy")

    # Call the API
    policy = privacy_policy_api.get()
    if policy is not None:
        policy.content = parse(policy.content)

    return render(
        request, "core_website_app/user/privacy-policy.html", context={"policy": policy}
    )


def terms_of_use(request):
    """Page that provides terms of use

    Parameters:
        request:

    Returns: Http Response
    """
    # Call the API
    terms = terms_of_use_api.get()
    if terms is not None:
        terms.content = parse(terms.content)

    return render(
        request, "core_website_app/user/terms-of-use.html", context={"terms": terms}
    )


def rules_of_behavior(request):
    """Page that provides the rules of behavior

    Parameters:
        request:

    Returns: Http Response
    """
    # Call the API
    rules_of_behavior_object = rules_of_behavior_api.get()
    if rules_of_behavior_object is not None:
        rules_of_behavior_object.content = parse(rules_of_behavior_object.content)

    return render(
        request,
        "core_website_app/user/rules_of_behavior.html",
        context={"rules_of_behavior": rules_of_behavior_object},
    )



""" Type of WebPages
"""
from core_main_app.commons.enums import WEB_PAGE_TYPES

WEB_PAGE_TYPES.update(
    {
        "terms_of_use": 0,
        "privacy_policy": 1,
        "help": 2,
        "rules_of_behavior": 4,
    }
)



""" Tests of the account request API
"""
from unittest.case import TestCase

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from mock import Mock, patch

from core_main_app.commons.exceptions import ApiError
from core_website_app.components.account_request import api as account_request_api
from core_website_app.components.account_request.models import AccountRequest


class TestsAccountRequestGet(TestCase):
    @patch(
        "core_website_app.components.account_request.models.AccountRequest.get_by_id"
    )
    def test_account_request_get_return_request_object(self, mock_get_by_id):
        # Arrange
        request_id = "1"
        mock_get_by_id.return_value = _create_account_request()
        # Act
        result = account_request_api.get(request_id)
        # Assert
        self.assertIsInstance(result, AccountRequest)

    @patch(
        "core_website_app.components.account_request.models.AccountRequest.get_by_id"
    )
    def test_account_request_get_raise_ApiError_if_request_not_found(
        self, mock_get_by_id
    ):
        # Arrange
        request_id = "1"
        mock_get_by_id.side_effect = Exception()
        # Act # Assert
        with self.assertRaises(ApiError):
            account_request_api.get(request_id)


class TestsAccountRequestAccept(TestCase):
    def setUp(self):
        self.account_request = _create_account_request()

    @patch("core_website_app.components.account_request.models.AccountRequest.delete")
    @patch("core_website_app.components.account_request.api._get_user_by_username")
    def test_account_request_accept_raise_ApiError_if_user_does_not_exist(
        self, mock_get_user_by_username, mock_delete
    ):
        # Arrange
        mock_get_user_by_username.side_effect = ObjectDoesNotExist()

        # Act # Assert
        with self.assertRaises(ApiError):
            account_request_api.accept(self.account_request)

    @patch("core_website_app.components.account_request.models.AccountRequest.delete")
    @patch("core_website_app.components.account_request.api._create_and_save_user")
    @patch("core_website_app.components.account_request.api._get_user_by_username")
    def test_account_request_accept_return_user(
        self, mock_get_by_username, mock_create_and_save, mock_delete
    ):
        # Arrange
        mock_user = Mock(spec=User)
        mock_user.id = 1
        mock_get_by_username.return_value = mock_user
        mock_create_and_save.return_value = mock_user
        mock_delete.return_value = None

        # Act
        result = account_request_api.accept(self.account_request)

        #  Assert
        self.assertIsInstance(result, User)


class TestsAccountRequestInsert(TestCase):
    def setUp(self):
        self.mock_account_request = _create_account_request()

    @patch("core_website_app.components.account_request.api._get_user_by_username")
    def test_account_request_insert_raise_ApiError_if_username_already_exist(
        self, mock_get_user_by_username
    ):
        # Arrange
        mock_user = Mock(spec=User)
        mock_user.username = "username"
        mock_get_user_by_username.return_value = mock_user

        # Act # Assert
        with self.assertRaises(ApiError):
            account_request_api.insert(mock_user)

    @patch("core_website_app.components.account_request.models.AccountRequest.save")
    @patch("core_website_app.components.account_request.api._get_user_by_username")
    def test_account_request_insert_return_request(
        self, mock_get_user_by_username, mock_save
    ):
        # Arrange
        mock_user = Mock(spec=User)
        mock_user.username = "username"
        mock_get_user_by_username.side_effect = ObjectDoesNotExist()
        mock_save.return_value = self.mock_account_request

        # Act
        result = account_request_api.insert(mock_user)

        # Assert
        self.assertIsInstance(result, AccountRequest)


def _create_account_request(username="username"):
    """
    Create an AccountRequest object using default parameters

    Parameters:
        username (str):

    Returns:
        AccountRequest object
    """
    return AccountRequest(username)



""" Rest views for the contact message API
"""
import logging

from django.http import Http404
from django.utils.decorators import method_decorator
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.views import APIView

import core_website_app.components.contact_message.api as contact_message_api
from core_main_app.commons import exceptions
from core_main_app.utils.decorators import api_staff_member_required
from core_website_app.rest.contact_message.serializers import ContactMessageSerializer

logger = logging.getLogger("core_website_app.rest.contact_message.views")


class ContactMessageList(APIView):
    """Create or get all Contact Message"""

    @method_decorator(api_staff_member_required())
    def get(self, request):
        """Get all Contact Message

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of contact messages
            - code: 500
              content: Internal server error
        """
        try:
            contact_message_list = contact_message_api.get_all()

            # Serialize object
            serializer = ContactMessageSerializer(contact_message_list, many=True)

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def post(self, request):
        """Create a message

        Parameters:

            {
                "name": "name",
                "content": "message",
                "email": "email"
            }

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: Contact message
            - code: 400
              content: Validation error / missing parameters
            - code: 500
              content: Internal server error
        """
        try:
            # Build serializer
            serializer = ContactMessageSerializer(data=request.data)

            # Validate message
            serializer.is_valid(True)

            # Save message
            serializer.save()

            # Return the serialized message
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class ContactMessageDetail(APIView):
    """Get or delete a Contact Message"""

    def get_object(self, pk):
        """Get Contact Message from db

        Args:

            pk: ObjectId

        Returns:

            Contact Message
        """
        try:
            return contact_message_api.get(pk)
        except exceptions.DoesNotExist:
            raise Http404

    @method_decorator(api_staff_member_required())
    def get(self, request, pk):
        """Retrieve a Contact Message

        Parameters:

            {
                "pk": "message_id"
            }

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content: Contact message
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            contact_message_object = self.get_object(pk)

            # Serialize object
            serializer = ContactMessageSerializer(contact_message_object)

            # Return response
            return Response(serializer.data)
        except Http404:
            content = {"message": "Contact message not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    @method_decorator(api_staff_member_required())
    def delete(self, request, pk):
        """Delete a Contact Message

        Parameters:

            {
                "pk": "message_id"
            }

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 204
              content: Deletion succeed
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            contact_message_object = self.get_object(pk)

            # delete object
            contact_message_api.delete(contact_message_object)

            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except Http404:
            content = {"message": "Data not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



"""
    Ajax views for the admin part
"""
import json

from django.contrib.admin.views.decorators import staff_member_required
from django.http import HttpResponse, HttpResponseBadRequest
from django.template import loader
from core_website_app.settings import (
    SERVER_URI,
    SEND_EMAIL_WHEN_ACCOUNT_REQUEST_IS_DENIED,
)
from xml_utils.commons.exceptions import HTMLError

import core_website_app.components.account_request.api as account_request_api
import core_website_app.components.contact_message.api as contact_message_api
from core_main_app.commons import exceptions as main_exceptions
from core_website_app.commons import exceptions
from xml_utils.html_tree.parser import parse_html
from core_main_app.templatetags.stripjs import stripjs


@staff_member_required
def accept_request(request):
    """
    Accepts a request and creates the user account
    :param request:
    :return:
    """
    try:
        request_id = request.POST["requestid"]
        account_request_from_api = account_request_api.get(request_id)
        account_request_api.accept(account_request_from_api)
        message = "Request Accepted"
    except main_exceptions.ApiError as error:
        raise exceptions.WebsiteAjaxError(str(error))
    except Exception as exception:
        raise exceptions.WebsiteAjaxError(str(exception))

    return HttpResponse(
        json.dumps({"message": message}), content_type="application/json"
    )


@staff_member_required
def deny_request(request):
    """
    Denies an account request
    :param request:
    :return:
    """

    try:
        email_params = None
        send_email = (
            request.POST.get("sendEmail") == "true"
            and SEND_EMAIL_WHEN_ACCOUNT_REQUEST_IS_DENIED
        )
        request_id = request.POST.get("requestid")
        email_subject = request.POST.get("emailParams[subject]")
        email_body = request.POST.get("emailParams[body]")

        if request_id and send_email and email_subject and email_body:

            # check the HTML syntax
            parse_html(email_body, "div")

            # check dangerous script injection
            if email_body != stripjs(email_body):
                return HttpResponseBadRequest("Unsafe HTML.")

            email_params = {
                "subject": email_subject,
                "body": email_body,
            }
        elif not request_id:
            raise ("Wrong parameters.")

        account_request_from_api = account_request_api.get(request_id)
        account_request_api.deny(account_request_from_api, send_email, email_params)
        message = "Request denied"

    except HTMLError as e:
        return HttpResponseBadRequest("HTML is not generated properly.")
    except main_exceptions.ApiError as error:
        raise exceptions.WebsiteAjaxError(str(error))
    except Exception as exception:
        raise exceptions.WebsiteAjaxError(str(exception))

    return HttpResponse(
        json.dumps({"message": message}), content_type="application/json"
    )


@staff_member_required
def remove_message(request):
    """
    Remove a message from the list of messages
    :param request:
    :return:
    """

    try:
        contact_message = contact_message_api.get(request.POST["messageid"])
        contact_message_api.delete(contact_message)
        message = "Message deleted"
    except exceptions.WebsiteAjaxError as error:
        message = str(error)
    except Exception as exception:
        message = str(exception)

    return HttpResponse(
        json.dumps({"message": message}), content_type="application/json"
    )


@staff_member_required
def get_deny_email_template(request):
    request_id = request.GET.get("requestid")

    if request_id:
        account_request = account_request_api.get(request_id)

        context = {
            "lastname": account_request.last_name,
            "firstname": account_request.first_name,
            "URI": SERVER_URI,
        }

        template = loader.get_template(
            "core_website_app/admin/email/request_account_denied.html"
        )
        content = template.render(context)

        return HttpResponse(
            json.dumps({"template": content}), content_type="application/json"
        )
    else:
        return HttpResponseBadRequest("Missing account request id.")


@staff_member_required
def account_request_count(request):
    """


    Returns:

    """
    return HttpResponse(
        json.dumps({"count": account_request_api.get_count()}),
        content_type="application/json",
    )


@staff_member_required
def contact_message_count(request):
    return HttpResponse(
        json.dumps({"count": contact_message_api.get_count()}),
        content_type="application/json",
    )



""" Terms of use api
"""

import core_main_app.components.web_page.api as web_page_api
from core_main_app.commons.exceptions import ApiError
from core_website_app.commons.enums import WEB_PAGE_TYPES

TERMS_PAGE_NAME = "terms_of_use"
TERMS_PAGE_TYPE = WEB_PAGE_TYPES[TERMS_PAGE_NAME]


def get():
    """Get the terms of use if exist

    Returns: Terms of use web page
    """
    return web_page_api.get(TERMS_PAGE_NAME)


def upsert(terms_of_use_page):
    """Post the terms of use

    Parameters:
        terms_of_use_page (WebPage): content of the web page

    Returns: Terms of use web page
    """
    if terms_of_use_page.type != TERMS_PAGE_TYPE:
        raise ApiError(
            "Webpage type not coherent (expected: %s; entered: %s)"
            % (str(TERMS_PAGE_TYPE), str(terms_of_use_page.type))
        )

    return web_page_api.upsert(terms_of_use_page)



""" Serializers used throughout the Contact Message Rest API
"""

from rest_framework_mongoengine.serializers import DocumentSerializer

from core_website_app.components.contact_message.models import ContactMessage


class ContactMessageSerializer(DocumentSerializer):
    """Represents the contact message serializer"""

    class Meta(object):
        model = ContactMessage
        fields = ["id", "name", "email", "content"]
        read_only_fields = ("id",)



""" Url router for the administration site
"""
from django.contrib import admin
from django.contrib.admin.views.decorators import staff_member_required
from django.urls import re_path

import core_website_app.components.help.api as help_api
import core_website_app.components.privacy_policy.api as privacy_policy_api
import core_website_app.components.terms_of_use.api as terms_of_use_api
import core_website_app.components.rules_of_behavior.api as rules_of_behavior_api
from core_main_app.components.web_page.models import WEB_PAGE_TYPES
from core_main_app.views.admin.views import WebPageView
from core_website_app.views.admin import views as admin_views, ajax as admin_ajax

admin_urls = [
    re_path(
        r"^user-requests$",
        admin_views.user_requests,
        name="core_website_app_user_requests",
    ),
    re_path(
        r"^accept_request",
        admin_ajax.accept_request,
        name="core_website_app_accept_user_request",
    ),
    re_path(
        r"^deny_request",
        admin_ajax.deny_request,
        name="core_website_app_deny_user_request",
    ),
    re_path(
        r"^get_deny_email_template",
        admin_ajax.get_deny_email_template,
        name="core_website_app_get_deny_email_template",
    ),
    re_path(
        r"^request_count",
        admin_ajax.account_request_count,
        name="core_website_app_request_count",
    ),
    re_path(
        r"^contact-messages$",
        admin_views.contact_messages,
        name="core_website_app_contact_messages",
    ),
    re_path(
        r"^remove_message",
        admin_ajax.remove_message,
        name="core_website_app_remove_contact_message",
    ),
    re_path(
        r"^message_count",
        admin_ajax.contact_message_count,
        name="core_website_app_message_count",
    ),
    re_path(
        r"^privacy-policy$",
        staff_member_required(
            WebPageView.as_view(
                api=privacy_policy_api,
                get_redirect="core_website_app/admin/privacy_policy.html",
                post_redirect="admin:core_website_app_privacy",
                web_page_type=WEB_PAGE_TYPES["privacy_policy"],
            )
        ),
        name="core_website_app_privacy",
    ),
    re_path(
        r"^terms-of-use$",
        staff_member_required(
            WebPageView.as_view(
                api=terms_of_use_api,
                get_redirect="core_website_app/admin/terms_of_use.html",
                post_redirect="admin:core_website_app_terms",
                web_page_type=WEB_PAGE_TYPES["terms_of_use"],
            )
        ),
        name="core_website_app_terms",
    ),
    re_path(
        r"^help$",
        staff_member_required(
            WebPageView.as_view(
                api=help_api,
                get_redirect="core_website_app/admin/help.html",
                post_redirect="admin:core_website_app_help",
                web_page_type=WEB_PAGE_TYPES["help"],
            )
        ),
        name="core_website_app_help",
    ),
    re_path(
        r"^rules_of_behavior$",
        staff_member_required(
            WebPageView.as_view(
                api=rules_of_behavior_api,
                get_redirect="core_website_app/admin/rules_of_behavior.html",
                post_redirect="admin:core_website_app_rules_of_behavior",
                web_page_type=WEB_PAGE_TYPES["rules_of_behavior"],
            )
        ),
        name="core_website_app_rules_of_behavior",
    ),
]

urls = admin.site.get_urls()
admin.site.get_urls = lambda: admin_urls + urls



""" Abstract view for Account Request
"""
from abc import ABCMeta, abstractmethod

from django.http import Http404
from django.utils.decorators import method_decorator
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.views import APIView

import core_website_app.components.account_request.api as account_request_api
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions
from core_main_app.commons.exceptions import ApiError
from core_main_app.utils.decorators import api_staff_member_required


class AbstractActionAccountRequest(APIView, metaclass=ABCMeta):
    """Action Account request"""

    def get_object(self, pk):
        """Get AccountRequest from db

        Args:

            pk: ObjectId

        Returns:

            Account Request
        """
        try:
            return account_request_api.get(pk)
        except exceptions.DoesNotExist:
            raise Http404

    @abstractmethod
    def perform(self, account_request_object):
        """Perform an action on the account request"""
        raise NotImplementedError("action method is not implemented.")

    @method_decorator(api_staff_member_required())
    def patch(self, request, pk):
        """Deny or Accept

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content: None
            - code: 400
              content: Validation error / bad request
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            account_request_object = self.get_object(pk)

            # Set current template
            self.perform(account_request_object)

            return Response(status=status.HTTP_200_OK)
        except Http404:
            content = {"message": "Account Request not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as access_error:
            content = {"message": str(access_error)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except ApiError as api_error:
            content = {"message": str(api_error)}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



""" Serializers used throughout the Account Request Rest API
"""

from django.contrib.auth.hashers import make_password
from django.contrib.auth.models import User
from rest_framework import serializers
from rest_framework_mongoengine.serializers import DocumentSerializer

import core_website_app.components.account_request.api as account_request_api
from core_website_app.components.account_request.models import AccountRequest


class AccountRequestSerializer(DocumentSerializer):
    """Represents the account request serializer"""

    class Meta(object):
        model = AccountRequest
        fields = ["id", "username", "first_name", "email", "date"]
        read_only_fields = (
            "id",
            "date",
        )


class UserSerializer(serializers.ModelSerializer):
    """Represents the user serializer"""

    class Meta(object):
        model = User
        fields = ["id", "username", "first_name", "last_name", "email", "password"]
        read_only_fields = ("id",)

    def create(self, validated_data):
        """Create and return a new `AccountRequest` instance, given the validated data."""
        user = User(
            username=validated_data["username"],
            first_name=validated_data["first_name"],
            last_name=validated_data["last_name"],
            email=validated_data["email"],
            password=make_password(validated_data["password"]),
            is_active=False,
        )

        return account_request_api.insert(user)



""" Form needed for the user part of everything
"""
from django import forms
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth.models import User
from captcha.fields import CaptchaField


class RequestAccountForm(UserCreationForm):
    """
    Form to request an account
    """

    username = forms.CharField(label="Username", max_length=100, required=True)
    firstname = forms.CharField(label="First Name", max_length=100, required=True)
    lastname = forms.CharField(label="Last Name", max_length=100, required=True)
    email = forms.EmailField(label="Email Address", max_length=100, required=True)
    captcha = CaptchaField()

    class Meta:
        model = User
        fields = (
            "username",
            "firstname",
            "lastname",
            "email",
            "password1",
            "password2",
        )


class ContactForm(forms.Form):
    """
    Form to contact the administrator
    """

    name = forms.CharField(label="Name", max_length=100, required=True)
    email = forms.EmailField(label="Email Address", max_length=100, required=True)
    message = forms.CharField(
        label="Message",
        widget=forms.Textarea(attrs={"class": "textarea_lock_h"}),
        required=True,
    )
    captcha = CaptchaField()



""" Url router for the Rest API
"""

from django.urls import re_path

import core_main_app.rest.web_page.views as web_page_views
import core_website_app.rest.account_request.views as account_request_views
import core_website_app.rest.contact_message.views as contact_message_views

urlpatterns = [
    re_path(
        r"^user-requests/$",
        account_request_views.AccountRequestList.as_view(),
        name="core_website_app_rest_account_request_list",
    ),
    re_path(
        r"^user-requests/(?P<pk>\w+)/$",
        account_request_views.AccountRequestDetail.as_view(),
        name="core_website_app_rest_account_request_detail",
    ),
    re_path(
        r"^user-requests/(?P<pk>\w+)/accept/$",
        account_request_views.AccountRequestAccept.as_view(),
        name="core_website_app_rest_account_request_accept",
    ),
    re_path(
        r"^user-requests/(?P<pk>\w+)/deny/$",
        account_request_views.AccountRequestDeny.as_view(),
        name="core_website_app_rest_account_request_deny",
    ),
    re_path(
        r"^messages/$",
        contact_message_views.ContactMessageList.as_view(),
        name="core_website_app_rest_message_list",
    ),
    re_path(
        r"^messages/(?P<pk>\w+)/$",
        contact_message_views.ContactMessageDetail.as_view(),
        name="core_website_app_rest_message_detail",
    ),
    re_path(
        r"^help/$",
        web_page_views.WebPageList.as_view(web_page_type="help"),
        name="core_website_app_rest_help_list",
    ),
    re_path(
        r"^privacy_policy/$",
        web_page_views.WebPageList.as_view(web_page_type="privacy_policy"),
        name="core_website_app_rest_privacy_policy_list",
    ),
    re_path(
        r"^terms_of_use/$",
        web_page_views.WebPageList.as_view(web_page_type="terms_of_use"),
        name="core_website_app_rest_terms_of_use_list",
    ),
]



""" Authentication tests for Account Request REST API
"""

from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status

import core_website_app.rest.account_request.views as account_request_views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_website_app.components.account_request.models import AccountRequest
from core_website_app.rest.account_request.serializers import (
    AccountRequestSerializer,
    UserSerializer,
)


class TestAccountRequestListGetPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            account_request_views.AccountRequestList.as_view(),
            create_mock_user("1", is_anonymous=True),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_get(
            account_request_views.AccountRequestList.as_view(),
            create_mock_user("1", is_anonymous=False),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(AccountRequest, "get_all")
    @patch.object(AccountRequestSerializer, "data")
    def test_is_staff_returns_http_200(self, account_serializer_data, account_get_all):
        response = RequestMock.do_request_get(
            account_request_views.AccountRequestList.as_view(),
            create_mock_user("1", is_staff=True),
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAccountRequestListPostPermission(SimpleTestCase):
    def setUp(self):
        self.mock_data = {
            "username": "username",
            "first_name": "first_name",
            "last_name": "last_name",
            "password": "password",
            "email": "email",
        }

    @patch.object(UserSerializer, "is_valid")
    @patch.object(UserSerializer, "save")
    def test_anonymous_returns_http_201(
        self, user_serializer_save, user_serializer_is_valid
    ):
        response = RequestMock.do_request_post(
            account_request_views.AccountRequestList.as_view(),
            create_mock_user("1", is_anonymous=True),
            data=self.mock_data,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(UserSerializer, "is_valid")
    @patch.object(UserSerializer, "save")
    def test_is_authenticated_returns_http_201(
        self, user_serializer_save, user_serializer_is_valid
    ):
        response = RequestMock.do_request_post(
            account_request_views.AccountRequestList.as_view(),
            create_mock_user("1", is_anonymous=False),
            data=self.mock_data,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(UserSerializer, "is_valid")
    @patch.object(UserSerializer, "save")
    def test_is_staff_returns_http_201(
        self, user_serializer_save, user_serializer_is_valid
    ):
        response = RequestMock.do_request_post(
            account_request_views.AccountRequestList.as_view(),
            create_mock_user("1", is_staff=True),
            data=self.mock_data,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestAccountRequestDetailGetPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            account_request_views.AccountRequestDetail.as_view(),
            create_mock_user("1", is_anonymous=True),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_get(
            account_request_views.AccountRequestDetail.as_view(),
            create_mock_user("1", is_anonymous=False),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(AccountRequest, "get_by_id")
    @patch.object(AccountRequestSerializer, "data")
    def test_is_staff_returns_http_200(
        self, account_serializer_data, account_get_by_id
    ):
        response = RequestMock.do_request_get(
            account_request_views.AccountRequestDetail.as_view(),
            create_mock_user("1", is_staff=True),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAccountRequestDenyGetPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            account_request_views.AccountRequestDeny.as_view(),
            create_mock_user("1", is_anonymous=True),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_patch(
            account_request_views.AccountRequestDeny.as_view(),
            create_mock_user("1", is_anonymous=False),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(AccountRequest, "get_by_id")
    @patch("core_website_app.components.account_request.api.deny")
    def test_is_staff_returns_http_200(self, account_api, account_get_by_id):
        response = RequestMock.do_request_patch(
            account_request_views.AccountRequestDeny.as_view(),
            create_mock_user("1", is_staff=True),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAccountRequestAcceptGetPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_patch(
            account_request_views.AccountRequestAccept.as_view(),
            create_mock_user("1", is_anonymous=True),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_patch(
            account_request_views.AccountRequestAccept.as_view(),
            create_mock_user("1", is_anonymous=False),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(AccountRequest, "get_by_id")
    @patch("core_website_app.components.account_request.api.accept")
    def test_is_staff_returns_http_200(self, account_api, account_get_by_id):
        response = RequestMock.do_request_patch(
            account_request_views.AccountRequestAccept.as_view(),
            create_mock_user("1", is_staff=True),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



"""Custom context processor
"""
from django.conf import settings


def domain_context_processor(request):
    return {
        "DISPLAY_NIST_HEADERS": settings.DISPLAY_NIST_HEADERS
        if hasattr(settings, "DISPLAY_NIST_HEADERS")
        else True,
    }



from django.urls import reverse
from menu import Menu, MenuItem

from core_website_app.settings import (
    DISPLAY_PRIVACY_POLICY_FOOTER,
    DISPLAY_TERMS_OF_USE_FOOTER,
    DISPLAY_CONTACT_FOOTER,
    DISPLAY_HELP_FOOTER,
    DISPLAY_RULES_OF_BEHAVIOR_FOOTER,
)

if DISPLAY_PRIVACY_POLICY_FOOTER:
    Menu.add_item(
        "footer",
        MenuItem("Privacy policy", reverse("core_website_app_privacy"), weight=1001),
    )
if DISPLAY_TERMS_OF_USE_FOOTER:
    Menu.add_item(
        "footer",
        MenuItem("Terms of use", reverse("core_website_app_terms"), weight=1002),
    )
if DISPLAY_CONTACT_FOOTER:
    Menu.add_item(
        "footer",
        MenuItem(
            "Contact", reverse("core_website_app_contact"), icon="envelope", weight=1003
        ),
    )
if DISPLAY_HELP_FOOTER:
    Menu.add_item(
        "footer",
        MenuItem(
            "Help",
            reverse("core_website_app_help"),
            icon="question-circle",
            weight=1004,
        ),
    )
if DISPLAY_RULES_OF_BEHAVIOR_FOOTER:
    Menu.add_item(
        "footer",
        MenuItem(
            "Rules of Behavior",
            reverse("core_website_app_rules_of_behavior"),
            icon="balance-scale",
            weight=1004,
        ),
    )

# Admin menus for website app
website_children = (
    MenuItem(
        "Privacy Policy", reverse("admin:core_website_app_privacy"), icon="user-secret"
    ),
    MenuItem("Terms of Use", reverse("admin:core_website_app_terms"), icon="file-alt"),
    MenuItem(
        "Help Page", reverse("admin:core_website_app_help"), icon="question-circle"
    ),
    MenuItem(
        "Rules of Behavior",
        reverse("admin:core_website_app_rules_of_behavior"),
        icon="balance-scale",
    ),
    MenuItem(
        "User requests",
        reverse("admin:core_website_app_user_requests"),
        icon="user-plus",
        item_count_url="admin:core_website_app_request_count",
    ),
    MenuItem(
        "Contact messages",
        reverse("admin:core_website_app_contact_messages"),
        icon="envelope",
        item_count_url="admin:core_website_app_message_count",
    ),
)

Menu.add_item(
    "admin", MenuItem("WEBSITE", None, children=website_children, weight=-10000)
)



SECRET_KEY = "fake-key"
SERVER_URI = "http://localhost"

INSTALLED_APPS = [
    # Django apps
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sites",
    "django.contrib.sessions",
    # Local app
    "tests",
]

LOGGING = {
    "version": 1,
    "disable_existing_loggers": True,
    "handlers": {
        "null": {
            "level": "DEBUG",
            "class": "logging.NullHandler",
        },
    },
    "loggers": {
        "django": {
            "handlers": ["null"],
            "propagate": True,
            "level": "WARN",
        },
        "django.db.backends": {
            "handlers": ["null"],
            "level": "DEBUG",
            "propagate": False,
        },
        "": {  # use 'MYAPP' to make it app specific
            "handlers": ["null"],
            "level": "DEBUG",
        },
    },
}



""" Privacy policy api
"""

import core_main_app.components.web_page.api as web_page_api
from core_main_app.commons.exceptions import ApiError
from core_website_app.commons.enums import WEB_PAGE_TYPES

PRIVACY_PAGE_NAME = "privacy_policy"
PRIVACY_PAGE_TYPE = WEB_PAGE_TYPES[PRIVACY_PAGE_NAME]


def get():
    """Get the privacy policy if exist

    Returns: privacy policy web page
    """
    return web_page_api.get(PRIVACY_PAGE_NAME)


def upsert(privacy_policy_page):
    """Post the privacy policy

    Parameters:
        privacy_policy_page (WebPage): WebPage for the privacy policy

    Returns: privacy policy web page
    """
    if privacy_policy_page.type != PRIVACY_PAGE_TYPE:
        raise ApiError(
            "Webpage type not coherent (expected: %s; actual %s"
            % (str(PRIVACY_PAGE_TYPE), str(privacy_policy_page.type))
        )

    return web_page_api.upsert(privacy_policy_page)



"""
    Core website exceptions
"""


class WebsiteAjaxError(Exception):
    """
    Exception raised by the Website package from ajax
    """

    def __init__(self, message):
        self.message = message



""" Tests of contact message API
"""
from unittest.case import TestCase

from mock import Mock, patch

from core_main_app.commons import exceptions
from core_website_app.components.contact_message import api as contact_message_api
from core_website_app.components.contact_message.models import ContactMessage


class TestsContactMessageGet(TestCase):
    """"""

    def setUp(self):
        self.message_id = 1

    @patch(
        "core_website_app.components.contact_message.models.ContactMessage.get_by_id"
    )
    def test_contact_message_get_by_id_return_object(self, mock_get_by_id):
        # Arrange
        mock_message = _create_mock_contact_message()
        mock_get_by_id.return_value = mock_message

        # Act
        result = contact_message_api.get(self.message_id)

        # Assert
        self.assertIsInstance(result, ContactMessage)

    @patch(
        "core_website_app.components.contact_message.models.ContactMessage.get_by_id"
    )
    def test_contact_message_get_raise_api_error_if_not_found(self, mock_get_by_id):
        # Arrange
        mock_get_by_id.side_effect = Exception()

        # Act # Assert
        with self.assertRaises(exceptions.ApiError):
            contact_message_api.get(self.message_id)


class TestContactMessageGetAll(TestCase):
    @patch("core_website_app.components.contact_message.models.ContactMessage.get_all")
    def test_contact_message_get_all_contains_contact_message_only(self, mock_get_all):
        # Arrange
        mock_message_1 = _create_mock_contact_message()
        mock_message_2 = _create_mock_contact_message(
            mock_pk=2,
            mock_name="message name 2",
            mock_email="e2@mail.com",
            mock_content="content message 2",
        )

        mock_get_all.return_value = [mock_message_1, mock_message_2]

        # Act
        result = contact_message_api.get_all()

        # Assert
        self.assertTrue(all(isinstance(item, ContactMessage) for item in result))


class TestsContactMessageDelete(TestCase):
    @patch("core_website_app.components.contact_message.models.ContactMessage.delete")
    def test_message_delete_raise_api_error_if_message_does_not_exist(
        self, mock_message_delete
    ):
        # Arrange
        mock_message = _create_contact_message()
        mock_message_delete.side_effect = Exception()

        # Act # Assert
        with self.assertRaises(exceptions.ApiError):
            contact_message_api.delete(mock_message)


class TestsContactMessageUpsert(TestCase):
    def setUp(self):
        self.mock_message = _create_contact_message()

    @patch("core_website_app.components.contact_message.models.ContactMessage.save")
    def test_message_upsert_return_message(self, mock_save):
        # Arrange
        mock_save.return_value = self.mock_message

        # Act
        result = contact_message_api.upsert(self.mock_message)

        # Assert
        self.assertIsInstance(result, ContactMessage)

    @patch("core_website_app.components.contact_message.models.ContactMessage.save")
    def test_message_upsert_raise_api_error_if_save_failed(self, mock_save):
        # Arrange
        mock_save.side_effect = Exception()

        # Act # Assert
        with self.assertRaises(exceptions.ApiError):
            contact_message_api.upsert(self.mock_message)


def _create_mock_contact_message(
    mock_pk=1,
    mock_name="message name",
    mock_email="sender@mail.com",
    mock_content="lorem ipsum dolor sit amet ",
):
    """

    :param mock_pk:
    :param mock_name:
    :param mock_email:
    :param mock_content:

    :return:
    """
    mock_message = Mock(spec=ContactMessage)
    mock_message.id = mock_pk
    mock_message.name = mock_name
    mock_message.email = mock_email
    mock_message.content = mock_content

    return mock_message


def _create_contact_message(
    pk=1,
    name="message name",
    email="sender@mail.com",
    content="lorem ipsum dolor sit amet ",
):
    """

    :param pk:
    :param name:
    :param email:
    :param content:

    :return:
    """

    return ContactMessage(pk=pk, name=name, email=email, content=content)



"""contact message API
"""
import logging

from core_main_app.commons import exceptions
from core_website_app.components.contact_message.models import ContactMessage

logger = logging.getLogger("core_website_app.components.contact_message.api")


def get_all():
    """List all messages

    Returns:

    """
    return ContactMessage.get_all()


def get_count():
    """Count number of contact messages currently in the database.

    Returns:
        int: number of contact messages
    """
    return len(get_all())


def get(message_id):
    """Get a message

    Args:
        message_id:

    Returns:

    """
    try:
        return ContactMessage.get_by_id(message_id)
    except Exception as e:
        logger.error(str(e))
        raise exceptions.ApiError("No message could be found with the given id.")


def upsert(contact_message):
    """Insert or update a given message

    Args:
        contact_message:

    Returns:

    """
    try:
        # save method return self
        return contact_message.save()
    except Exception as e:
        logger.error(str(e))
        raise exceptions.ApiError("Save message failed")


def delete(contact_message):
    """Delete a message

    Args:
        contact_message:

    Returns:

    """
    try:
        contact_message.delete()
    except Exception as e:
        logger.error(str(e))
        raise exceptions.ApiError("Impossible to delete contact message.")



""" Settings for core_website_app

Settings with the following syntax can be overwritten at the project level:
SETTING_NAME = getattr(settings, "SETTING_NAME", "Default Value")
"""
from django.conf import settings

if not settings.configured:
    settings.configure()

SERVER_URI = getattr(settings, "SERVER_URI", "http://localhost")

INSTALLED_APPS = ("core_main_app",)

DISPLAY_NIST_HEADERS = getattr(settings, "DISPLAY_NIST_HEADERS", False)
""" boolean: display the NIST header
"""
DISPLAY_PRIVACY_POLICY_FOOTER = getattr(
    settings, "DISPLAY_PRIVACY_POLICY_FOOTER", False
)
""" boolean: display the privacy policy link in the footer
"""
DISPLAY_TERMS_OF_USE_FOOTER = getattr(settings, "DISPLAY_TERMS_OF_USE_FOOTER", False)
""" boolean: display the terms of use link in the footer
"""
DISPLAY_CONTACT_FOOTER = getattr(settings, "DISPLAY_CONTACT_FOOTER", False)
""" boolean: display the contact link in the footer
"""
DISPLAY_HELP_FOOTER = getattr(settings, "DISPLAY_HELP_FOOTER", False)
""" boolean: display the help link in the footer
"""
DISPLAY_RULES_OF_BEHAVIOR_FOOTER = getattr(
    settings, "DISPLAY_RULES_OF_BEHAVIOR_FOOTER", False
)
""" boolean: display the rules of behavior link in the footer
"""
SEND_EMAIL_WHEN_ACCOUNT_REQUEST_IS_ACCEPTED = getattr(
    settings, "SEND_EMAIL_WHEN_ACCOUNT_REQUEST_IS_ACCEPTED", False
)
""" boolean: send an email when an account is accepted
"""
SEND_EMAIL_WHEN_ACCOUNT_REQUEST_IS_DENIED = getattr(
    settings, "SEND_EMAIL_WHEN_ACCOUNT_REQUEST_IS_DENIED", False
)
""" boolean: send an email when an account is denied
"""
EMAIL_DENY_SUBJECT = getattr(settings, "EMAIL_DENY_SUBJECT", "Account request denied")
""" boolean: subject of deny email
"""



""" The API contains the available function to access, create, edit and delete the account requests
"""
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist

import core_main_app.utils.notifications.mail as send_mail_api
from core_main_app.commons.exceptions import ApiError
from core_website_app.components.account_request.models import AccountRequest
from core_website_app.settings import (
    SERVER_URI,
    SEND_EMAIL_WHEN_ACCOUNT_REQUEST_IS_ACCEPTED,
    EMAIL_DENY_SUBJECT,
)


def get_all():
    """List of opened account requests

    Returns:

        List of all requests
    """
    return AccountRequest.get_all()


def get_count():
    """Count number of account request currently in the database

    Returns:

        number of account requests
    """
    return get_all().count()


def get(account_request_id):
    """Get an account request given its primary key

    Args:

        account_request_id: Primary key of the request

    Returns:

        Account request
    """
    try:
        return AccountRequest.get_by_id(account_request_id)
    except:
        raise ApiError("No request could be found with the given id.")


def insert(user):
    """Create a new request

    Args:

        user: Django User

    Returns:

        New account request
    """
    try:
        # check if a user with the same username exists
        _get_user_by_username(user.username)
        raise ApiError("A user with the same username already exists.")
    except ObjectDoesNotExist:
        user.save()

        # Create the account request and save it
        account_request = AccountRequest(
            username=user.username,
            first_name=user.first_name,
            last_name=user.last_name,
            email=user.email,
        )

        context = {"URI": SERVER_URI}
        template_path = "core_website_app/admin/email/request_account_for_admin.html"
        send_mail_api.send_mail_to_administrators(
            subject="New Account Request",
            path_to_template=template_path,
            context=context,
        )
        return account_request.save()


def accept(account_request):
    """Accept an account request

    Args:

        account_request: Primary key of the request

    Returns:

    """
    user = None
    try:
        # check if a user with the same username exists
        user = _get_user_by_username(account_request.username)
        user.is_active = True
        user.save()

        if SEND_EMAIL_WHEN_ACCOUNT_REQUEST_IS_ACCEPTED:
            # FIXME send_mail should use a User object
            context = {
                "lastname": account_request.last_name,
                "firstname": account_request.first_name,
                "URI": SERVER_URI,
            }

            send_mail_api.send_mail_from_template(
                subject="Account approved",
                path_to_template="core_website_app/admin/email/request_account_approved.html",
                context=context,
                recipient_list=[account_request.email],
            )
    finally:
        # delete the user request
        account_request.delete()
        if user is not None:
            return user
        else:
            raise ApiError("User does not exist")


def deny(account_request, send_email=True, email_params=None):
    """Delete an account request

    Args:

        account_request: Primary key of the request
        send_email: boolean to indicate if we have to send an email
        email_params: { subject: <email subject>, body: <email body> }

    Returns:

    """
    user = None
    try:
        # check if a user with the same username exists
        user = _get_user_by_username(account_request.username)
        user.delete()
    finally:
        # delete the user request
        account_request.delete()

        if send_email:
            # create the context for the email
            account_request_email = account_request.email
            context = {
                "lastname": account_request.last_name,
                "firstname": account_request.first_name,
                "URI": SERVER_URI,
            }

            inline_template = email_params.get("body", None)

            if inline_template:
                send_mail_api.send_mail(
                    recipient_list=[account_request_email],
                    subject=email_params.get("subject", EMAIL_DENY_SUBJECT),
                    body=inline_template,
                )
            else:
                send_mail_api.send_mail_from_template(
                    subject=email_params.get("subject", EMAIL_DENY_SUBJECT),
                    recipient_list=[account_request_email],
                    path_to_template="core_website_app/admin/email/request_account_denied.html",
                    context=context,
                )
        if user is not None:
            return
        else:
            raise ApiError("User does not exist")


def _get_user_by_username(username):
    """Returns a user given its username

    Args:

        username: Given username

    Returns:

        User
    """
    return User.objects.get(username=username)


def _get_user_by_id(user_id):
    """Returns a user given its primary key

    Args:

        user_id: Given user id

    Returns:

        User
    """
    return User.objects.get(pk=user_id)


def _create_and_save_user(username, password, first_name, last_name, email):
    """Save a user with the given parameters

    Args:

        username: Given user name
        password: Given password
        first_name: Given first name
        last_name: Given last name
        email: Given email

    Returns:

        User
    """
    user = User.objects.create_user(
        username=username,
        password=password,
        first_name=first_name,
        last_name=last_name,
        email=email,
    )
    # Have to be called separately because save from django object return nothing
    user.save()
    return user



""" Url router for the main application
"""
from django.conf.urls import include
from django.urls import re_path

from core_website_app.views.user import views as user_views

urlpatterns = [
    re_path("captcha/", include("captcha.urls")),
    re_path(
        r"^account-request",
        user_views.request_new_account,
        name="core_website_app_account_request",
    ),
    re_path(r"^contact", user_views.contact, name="core_website_app_contact"),
    re_path(r"^help", user_views.help_page, name="core_website_app_help"),
    re_path(r"^privacy", user_views.privacy_policy, name="core_website_app_privacy"),
    re_path(r"^terms", user_views.terms_of_use, name="core_website_app_terms"),
    re_path(
        r"^rules-of-behavior",
        user_views.rules_of_behavior,
        name="core_website_app_rules_of_behavior",
    ),
    re_path(r"^website/", include("core_website_app.rest.urls")),
]



""" Rules of behavior page api
"""

import core_main_app.components.web_page.api as web_page_api
from core_main_app.commons.exceptions import ApiError
from core_website_app.commons.enums import WEB_PAGE_TYPES

RULES_OF_BEHAVIOR_PAGE_NAME = "rules_of_behavior"
RULES_OF_BEHAVIOR_PAGE_TYPE = WEB_PAGE_TYPES[RULES_OF_BEHAVIOR_PAGE_NAME]


def get():
    """Get the help if exist

    Returns: rules of behavior web page
    """
    return web_page_api.get(RULES_OF_BEHAVIOR_PAGE_NAME)


def upsert(rules_of_behavior_page):
    """Post the rules of behavior

    Parameters:
        rules_of_behavior_page (WebPage): Webpage for the rules of behavior

    Returns: rules_of_behavior_page web page
    """
    if rules_of_behavior_page.type != RULES_OF_BEHAVIOR_PAGE_TYPE:
        raise ApiError(
            "Webpage type not coherent (expected: %s; actual %s"
            % (str(RULES_OF_BEHAVIOR_PAGE_TYPE), str(rules_of_behavior_page.type))
        )

    return web_page_api.upsert(rules_of_behavior_page)



""" Help page api
"""

import core_main_app.components.web_page.api as web_page_api
from core_main_app.commons.exceptions import ApiError
from core_website_app.commons.enums import WEB_PAGE_TYPES

HELP_PAGE_NAME = "help"
HELP_PAGE_TYPE = WEB_PAGE_TYPES[HELP_PAGE_NAME]


def get():
    """Get the help if exist

    Returns: help web page
    """
    return web_page_api.get(HELP_PAGE_NAME)


def upsert(help_page):
    """Post the help

    Parameters:
        help_page (WebPage): Webpage for the help

    Returns: help web page
    """
    if help_page.type != HELP_PAGE_TYPE:
        raise ApiError(
            "Webpage type not coherent (expected: %s; actual %s"
            % (str(HELP_PAGE_TYPE), str(help_page.type))
        )

    return web_page_api.upsert(help_page)



""" Contact messages models
"""

from django_mongoengine import fields, Document
from mongoengine import errors as mongoengine_errors

from core_main_app.commons import exceptions


class ContactMessage(Document):
    """Represents a message sent via the Contact form"""

    name = fields.StringField(max_length=100)
    email = fields.EmailField()
    content = fields.StringField()

    @staticmethod
    def get_by_id(message_id):
        """Get a message using its primary key

        Args:
            message_id:

        Returns:
        """
        try:
            return ContactMessage.objects().get(pk=str(message_id))
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_all():
        """Get all messages

        Returns:
        """
        return ContactMessage.objects.all()



""" Authentication tests for contact Message REST API
"""

from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status

import core_website_app.rest.contact_message.views as contact_message_views
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_website_app.components.contact_message.models import ContactMessage
from core_website_app.rest.contact_message.serializers import ContactMessageSerializer


class TestContactMessageListGetPermission(SimpleTestCase):
    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            contact_message_views.ContactMessageList.as_view(),
            create_mock_user("1", is_anonymous=True),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_get(
            contact_message_views.ContactMessageList.as_view(),
            create_mock_user("1", is_anonymous=False),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(ContactMessage, "get_all")
    @patch.object(ContactMessageSerializer, "data")
    def test_is_staff_returns_http_200(self, account_serializer_data, account_get_all):
        account_get_all.return_value = {}
        account_serializer_data.return_value = True

        response = RequestMock.do_request_get(
            contact_message_views.ContactMessageList.as_view(),
            create_mock_user("1", is_staff=True),
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestContactMessageListPostPermission(SimpleTestCase):
    def setUp(self):
        self.mock_account_request = ContactMessage(
            name="mock", content="mock", email="mock@mock.com"
        )
        self.mock_data = {"name": "name", "content": "message", "email": "email"}

    @patch.object(ContactMessageSerializer, "is_valid")
    @patch.object(ContactMessageSerializer, "save")
    @patch.object(ContactMessageSerializer, "data")
    def test_anonymous_returns_http_201(
        self,
        contact_serializer_data,
        contact_serializer_save,
        contact_serializer_is_valid,
    ):
        response = RequestMock.do_request_post(
            contact_message_views.ContactMessageList.as_view(),
            create_mock_user("1", is_anonymous=True),
            data=self.mock_data,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(ContactMessageSerializer, "is_valid")
    @patch.object(ContactMessageSerializer, "save")
    @patch.object(ContactMessageSerializer, "data")
    def test_is_authenticated_returns_http_201(
        self,
        contact_serializer_data,
        contact_serializer_save,
        contact_serializer_is_valid,
    ):
        response = RequestMock.do_request_post(
            contact_message_views.ContactMessageList.as_view(),
            create_mock_user("1", is_anonymous=False),
            data=self.mock_data,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(ContactMessageSerializer, "is_valid")
    @patch.object(ContactMessageSerializer, "save")
    @patch.object(ContactMessageSerializer, "data")
    def test_is_staff_returns_http_201(
        self,
        contact_serializer_data,
        contact_serializer_save,
        contact_serializer_is_valid,
    ):
        response = RequestMock.do_request_post(
            contact_message_views.ContactMessageList.as_view(),
            create_mock_user("1", is_staff=True),
            data=self.mock_data,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestAContactMessageDetailGetPermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_get(
            contact_message_views.ContactMessageDetail.as_view(),
            create_mock_user("1", is_anonymous=True),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_get(
            contact_message_views.ContactMessageDetail.as_view(),
            create_mock_user("1", is_anonymous=False),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(ContactMessage, "get_by_id")
    @patch.object(ContactMessageSerializer, "data")
    def test_is_staff_returns_http_200(
        self, contact_serializer_data, contact_get_by_id
    ):
        response = RequestMock.do_request_get(
            contact_message_views.ContactMessageDetail.as_view(),
            create_mock_user("1", is_staff=True),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestAContactMessageDetailDeletePermission(SimpleTestCase):
    def setUp(self):
        self.fake_id = "507f1f77bcf86cd799439011"

    def test_anonymous_returns_http_403(self):
        response = RequestMock.do_request_delete(
            contact_message_views.ContactMessageDetail.as_view(),
            create_mock_user("1", is_anonymous=True),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_is_authenticated_returns_http_403(self):
        response = RequestMock.do_request_delete(
            contact_message_views.ContactMessageDetail.as_view(),
            create_mock_user("1", is_anonymous=False),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(ContactMessage, "get_by_id")
    @patch.object(ContactMessageSerializer, "data")
    @patch("core_website_app.components.contact_message.api.delete")
    def test_is_staff_returns_http_200(
        self, contact_api, contact_serializer_data, contact_get_by_id
    ):
        response = RequestMock.do_request_delete(
            contact_message_views.ContactMessageDetail.as_view(),
            create_mock_user("1", is_staff=True),
            param={"pk": self.fake_id},
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)



""" App bundling basic functionalities for a minimal working website. Included in this app are the following:
    * Account requests: Request a new account to the administrator
    * Contact messages: Send private messages to the administrator
    * Help, Privacy policy and Terms of use: Special pages for the website

Authors:
    * Adrien Catel (adrien.catel@nist.gov)
    * Philippe Dessauw (philippe.dessauw@nist.gov)
    * Guillaume Sousa (guillaume.sousa@nist.gov)
    * Sharief Youssef (sharief.youssef@nist.gov)

Sponsor:
    Alden Dima (alden.dima@nist.gov)

    | Information Systems Group
    | Software and Systems Division
    | Information Technology Laboratory
    | National Institute of Standards and Technology
    | http://www.nist.gov/itl/ssd/is
"""



""" rest api views
"""
import logging

from django.http import Http404
from django.utils.decorators import method_decorator
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.views import APIView

import core_website_app.components.account_request.api as account_request_api
from core_main_app.commons import exceptions
from core_main_app.utils.decorators import api_staff_member_required
from core_website_app.rest.account_request.abstract_views import (
    AbstractActionAccountRequest,
)
from core_website_app.rest.account_request.serializers import (
    AccountRequestSerializer,
    UserSerializer,
)

logger = logging.getLogger("core_website_app.rest.account_request.views")


class AccountRequestList(APIView):
    """Create or get all Account Request"""

    @method_decorator(api_staff_member_required())
    def get(self, request):
        """Get all account requests

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of account requests
            - code: 400
              content: Validation error
        """
        try:
            account_request_list = account_request_api.get_all()

            # Serialize object
            serializer = AccountRequestSerializer(account_request_list, many=True)

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def post(self, request):
        """Create a new account request

        Parameters:

            {
                "username": "username",
                "first_name": "first_name",
                "last_name": "last_name",
                "password": "password",
                "email": "email"
            }

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: Account Request
            - code: 400
              content: Validation error / missing parameters
        """
        try:
            # Build serializer
            serializer = UserSerializer(data=request.data)

            # Validate request
            serializer.is_valid(True)

            # Save request
            account_request = serializer.save()

            # Account request serializer
            account_request_serializer = AccountRequestSerializer(account_request)

            # Return the serialized user request
            return Response(
                account_request_serializer.data, status=status.HTTP_201_CREATED
            )
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class AccountRequestDetail(APIView):
    """Get an Account Request"""

    def get_object(self, pk):
        """Get Account Request from db

        Args:

            pk: ObjectId

        Returns:

            Account Request
        """
        try:
            return account_request_api.get(pk)
        except exceptions.DoesNotExist:
            raise Http404

    @method_decorator(api_staff_member_required())
    def get(self, request, pk):
        """Retrieve a Account Request

        Parameters:

            {
                "pk": "account_request_id"
            }

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:

            - code: 200
              content: Account Request
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            account_request_object = self.get_object(pk)

            # Serialize object
            serializer = AccountRequestSerializer(account_request_object)

            # Return response
            return Response(serializer.data)
        except Http404:
            content = {"message": "Account request not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class AccountRequestDeny(AbstractActionAccountRequest):
    """Deny an Account Request"""

    def perform(self, account_request_object):
        """Deny an Account Request

        Args:

            account_request_object: account_request
        """
        account_request_api.deny(account_request_object)


class AccountRequestAccept(AbstractActionAccountRequest):
    """Accept an Account Request"""

    def perform(self, account_request_object):
        """Accept an Account Request

        Args:

            account_request_object: account_request
        """
        account_request_api.accept(account_request_object)



""" Account requests model
"""
import datetime

from django_mongoengine import fields, Document
from mongoengine import errors as mongoengine_errors

from core_main_app.commons import exceptions


class AccountRequest(Document):
    """Represents a request sent by an user to get an account"""

    username = fields.StringField(blank=False)  #: Username associated with the request
    first_name = fields.StringField(blank=False)
    last_name = fields.StringField(blank=False)
    email = fields.StringField(blank=False)
    date = fields.DateTimeField(default=datetime.datetime.now, blank=False)

    @staticmethod
    def get_by_id(request_id):
        """Get a request given its primary key

        Parameters:
            request_id (str): Primary key of the request

        Returns:
            Request object corresponding to the given id
        """
        try:
            return AccountRequest.objects().get(pk=str(request_id))
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_all():
        """Get all Account Request

        Returns:

        """
        return AccountRequest.objects.all()



""" Admin views
"""
from django.contrib.admin.views.decorators import staff_member_required
from django.template import loader

import core_website_app.components.account_request.api as account_request_api
import core_website_app.components.contact_message.api as contact_message_api
from core_main_app.utils.rendering import admin_render as render
from core_website_app.settings import (
    EMAIL_DENY_SUBJECT,
    SEND_EMAIL_WHEN_ACCOUNT_REQUEST_IS_DENIED,
)


@staff_member_required
def user_requests(request):
    """Page that allows to accept or deny user requests

    Args:
        request:

    Returns:
    """
    # Call the API
    requests = account_request_api.get_all()

    assets = {
        "js": [
            {"path": "core_website_app/admin/js/user_requests.js", "is_raw": False},
        ],
    }

    modals = [
        "core_website_app/admin/account_requests/modals/deny_request.html",
    ]

    return render(
        request,
        "core_website_app/admin/user_requests.html",
        assets=assets,
        modals=modals,
        context={
            "requests": requests,
            "send_email_when_account_request_is_denied": SEND_EMAIL_WHEN_ACCOUNT_REQUEST_IS_DENIED,
            "default_email_subject": EMAIL_DENY_SUBJECT,
        },
    )


@staff_member_required
def contact_messages(request):
    """List messages from the contact page

    Args:
        request:

    Returns:
    """

    # Call the API
    messages_contact = contact_message_api.get_all()

    assets = {
        "js": [
            {"path": "core_website_app/admin/js/messages.js", "is_raw": False},
        ],
        "css": ["core_website_app/admin/css/messages.css"],
    }

    modals = [
        "core_website_app/admin/contact_messages/modals/delete_message.html",
    ]

    return render(
        request,
        "core_website_app/admin/contact_messages.html",
        assets=assets,
        modals=modals,
        context={"contacts": messages_contact},
    )
